package cz.kebrt.unitcheck.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * The main configuration class of UnitCheck. The required configuration parameters are
 * <ul>
 * <li>directory with UnitCheck libraries and dependencies
 * <li>at least one test
 * </ul>
 *
 * @author michal
 */
public class Config {

    private List<TestConfig> tests;
    private String libdir;

    private String jpfConfigFile = null;
    private boolean haltOnError = true;
    private boolean filterJUnitTrace = true;

    private List<String> vmClasspath = new ArrayList<String>();
    private List<String> vmSourcepath = new ArrayList<String>();

    private Map<String, String> jpfProperties = new HashMap<String, String>();

    /**
     * Checks that all required parameters are set.
     *
     * @throws ConfigException some required parameter is not set
     */
    public void check() throws ConfigException {
        if (tests == null || tests.size() == 0) {
            throw new ConfigException("Test class not set.");
        }
        if (libdir == null) {
            throw new ConfigException(String.format("Unitcheck lib directory not set."));
        }
    }

    /**
     * Returns <code>true</code> when the JUnit and UnitCheck internal code listings
     * should be omitted from error traces of test failures.
     *
     * @return <code>true</code> when the JUnit and UnitCheck internal code listings
     *         should be omitted from error traces
     */
    public boolean isFilterJUnitTrace() {
        return filterJUnitTrace;
    }

    /**
     * Sets whether the JUnit and UnitCheck internal code listings
     * should be omitted from error traces of test failures.
     *
     * @param filterJUnitTrace <code>true</code> when yes
     */
    public void setFilterJUnitTrace(boolean filterJUnitTrace) {
        this.filterJUnitTrace = filterJUnitTrace;
    }

    /**
     * Returns the path to JPF configuration file.
     *
     * @return the path to JPF configuration file (may be <code>null</code>
     */
    public String getJPFConfigFile() {
        return jpfConfigFile;
    }

    /**
     * Sets the path to JPF configuration file.
     *
     * @param configFile the path to JPF configuration file
     */
    public void setJPFConfigFile(String configFile) {
        jpfConfigFile = configFile;
    }

    /**
     * Returns <code>true</code> when UnitCheck should stop checking after
     * a test failure occurs.
     *
     * @return <code>true</code> when UnitCheck should stop checking after
     *         a test failure occurs.
     */
    public boolean isHaltOnError() {
        return haltOnError;
    }

    /**
     * Sets whether UnitCheck should stop checking after
     * a test failure occurs.
     *
     * @param haltOnError <code>true</code> when yes
     */
    public void setHaltOnError(boolean haltOnError) {
        this.haltOnError = haltOnError;
    }

    /**
     * Sets classpath used for running tests.
     *
     * @param classpath classpath used for running tests
     */
    public void setVMClasspath(List<String> classpath) {
        this.vmClasspath = classpath;
    }

    /**
     * Returns classpath used for running tests.
     *
     * @return classpath used for running tests
     */
    public List<String> getVMClasspath() {
        return vmClasspath;
    }

    /**
     * Sets sourcepath used for running tests.
     *
     * @param sourcepath sourcepath used for running tests
     */
    public void setVMSourcepath(List<String> sourcepath) {
        this.vmSourcepath = sourcepath;
    }

    /**
     * Returns sourcepath (paths to source files of tests) used for running tests.
     *
     * @return sourcepath used for running tests
     */
    public List<String> getVMSourcepath() {
        return vmSourcepath;
    }

    /**
     * Sets a list of tests for checking.
     *
     * @param tests list of tests for checking
     */
    public void setTests(List<TestConfig> tests) {
        this.tests = tests;
    }

    /**
     * Returns a list of tests for checking.
     *
     * @return a list of tests for checking
     */
    public List<TestConfig> getTests() {
        return tests;
    }

    /**
     * Sets exactly one test for checking.
     *
     * @param test test for checking
     */
    public void setOneTest(TestConfig test) {
        tests = new ArrayList<TestConfig>(1);
        tests.add(test);
    }

    /**
     * Sets JPF properties. The properties will override default JPF properties.
     *
     * @param props map of JPF properties
     */
    public void setJPFProperties(Map<String, String> props) {
        this.jpfProperties = props;
    }

    /**
     * Returns JPF properties.
     *
     * @return map of JPF properties
     */
    public Map<String, String> getJPFProperties() {
        return jpfProperties;
    }

    /**
     * Sets directory of UnitCheck libraries and other dependencies.
     *
     * @param dir directory of UnitCheck libraries and other dependencies
     */
    public void setLibDir(String dir) {
        this.libdir = dir;
    }

    /**
     * Returns directory of UnitCheck libraries and other dependencies.
     * @return directory of UnitCheck libraries and other dependencies
     */
    public String getLibDir() {
        return libdir;
    }

}
