package cz.kebrt.unitcheck.jpf;

import gov.nasa.jpf.jvm.ChoiceGenerator;
import gov.nasa.jpf.jvm.ClassInfo;
import gov.nasa.jpf.jvm.MethodInfo;
import gov.nasa.jpf.jvm.Path;
import gov.nasa.jpf.jvm.Step;
import gov.nasa.jpf.jvm.Transition;
import gov.nasa.jpf.jvm.bytecode.Instruction;

import java.util.HashMap;
import java.util.Map;

import cz.kebrt.unitcheck.api.data.ChoiceGeneratorHolder;
import cz.kebrt.unitcheck.api.data.InstructionHolder;
import cz.kebrt.unitcheck.api.data.PathHolder;
import cz.kebrt.unitcheck.api.data.StepHolder;
import cz.kebrt.unitcheck.api.data.TransitionHolder;
import cz.kebrt.unitcheck.config.Config;
import cz.kebrt.unitcheck.junit.JUnitUtil;

/**
 * Converts Path structure provided by JPF into PathHolder that is exposed
 * to higher levels of UnitCheck. PathHolder provides only reader methods
 * (original Path allows to modify the underlying data) and adds a support
 * for bidirectional traversal of all transitions (and also transitions
 * executed by each thread).
 *
 * @author michal
 */
public final class PathConvertor {

    private static final int STEP_MAX_COUNT = 10000;

    private PathConvertor() { }

    /**
     * Transforms JPF Path structure into PathHolder structure.
     *
     * @param path   JPF Path structure
     * @param config UnitCheck configuration
     * @return PathHolder structure for the corresponding Path structure
     */
    public static PathHolder convert(Path path, Config config) {
        // keeps the last executed transition for each thread
        Map<Integer, TransitionHolder> lastInThread
            = new HashMap<Integer, TransitionHolder>();
        PathHolder ph = new PathHolder();
        TransitionHolder prev = null;
        int i = 0;
        for (Transition t : path) {
            TransitionHolder th = createTransition(t, prev, lastInThread, i);
            ph.addTransition(th);
            prev = th;
            createSteps(t, th, config);
            ++i;
        }
        return ph;
    }

    private static void createSteps(Transition t, TransitionHolder th, Config config) {
        Step s = null;
        if (t.getStepCount() > STEP_MAX_COUNT) {
            s = t.getStep(t.getStepCount() - STEP_MAX_COUNT);
        } else if (t.getStepCount() > 0) {
            s = t.getStep(0);
        }
        while (s != null) {
            if (!config.isFilterJUnitTrace()
                    || !JUnitUtil.isJUnitCode(s.getLocationString())) {
                StepHolder sh = new StepHolder(s.getLineString(), s.getLocationString(),
                        createInstruction(s.getInstruction()));
                th.addStep(sh);
            }
            s = s.getNext();
        }
    }

    private static TransitionHolder createTransition(Transition t, TransitionHolder prev,
            Map<Integer, TransitionHolder> lastInThread, int no) {
        ChoiceGeneratorHolder choiceGen = createChoiceGenerator(t.getChoiceGenerator());
        TransitionHolder prevInThread = lastInThread.get(t.getThreadIndex());
        TransitionHolder th = new TransitionHolder(no, t.getThreadIndex(),
                choiceGen, prevInThread, prev);
        if (prev != null) {
            prev.setNext(th);
        }
        if (prevInThread != null) {
            prevInThread.setNextInThread(th);
        }
        lastInThread.put(t.getThreadIndex(), th);
        return th;
    }

    private static InstructionHolder createInstruction(Instruction instruction) {
        String className = null;
        String methodName = null;
        MethodInfo mi = instruction.getMethodInfo();
        if (mi != null) {
            ClassInfo mci = mi.getClassInfo();
            if (mci != null) {
                className = mci.getName();
            }
            methodName = mi.getUniqueName();
        }

        return new InstructionHolder(className, methodName, instruction.toString());
    }

    private static ChoiceGeneratorHolder createChoiceGenerator(
            ChoiceGenerator<?> choice) {
        return new ChoiceGeneratorHolder(choice.getClass().getCanonicalName(), choice
                .toString(), choice.getProcessedNumberOfChoices(), choice
                .getTotalNumberOfChoices(), choice.getInsn().getSourceLine());
    }

}
